# -*- coding:utf-8 -*-
from django.shortcuts import render
from django.http import JsonResponse
from ajaxlte.utils import Codec
from django.views.generic import View
from django.forms import modelform_factory
from django.contrib.admin.utils import flatten
from django.template.loader import render_to_string
from django.core.paginator import Paginator
from django.db.models import Q
from functools import reduce
import operator
from django.db.models.fields.related_descriptors import ForwardManyToOneDescriptor
from django.db.models.query_utils import DeferredAttribute
from django.contrib.auth.decorators import login_required
from django.db import IntegrityError
from django.urls import path


class AjaxSite:
    logo_url = ""
    proyect_name = ""
    spinner = ""
    name_space = ""
    root_url = ""
    login_url = ""
    menu = {}
    urlpatterns = []

    @classmethod
    def set_index(cls, index_view):
        cls.urlpatterns.append(path('', index_view.as_view(), name="index"))

    @classmethod
    def get_root_url(cls):
        if cls.root_url == "":
            return ""
        else:
            return "/%s" % cls.root_url

    @classmethod
    def add_pill(cls, module):
        cls.menu[module] = {
            'name': module,
            'links': []
        }

    @classmethod
    def register(cls, class_view, module):
        url = class_view.model.__name__ + "/"
        cls.urlpatterns.append(path(url, class_view.as_view(), name=class_view.model.__name__))
        cls.menu[module]['links'].append({
            'name': class_view.model._meta.verbose_name_plural,
            'url': cls.get_root_url() + url
        })

    @classmethod
    def branding(cls):
        return {'logo': cls.logo_url, 'proyect_name': cls.proyect_name, 'spinner': cls.spinner}


class Index(View):
    template_name = "ajaxlte/index.html"
    site = None

    @classmethod
    def as_view(cls, **initkwars):
        return login_required(super().as_view(**initkwars), login_url=cls.site.login_url)

    def get(self, request):
        return render(request, self.template_name, {
            'branding': self.site.branding(),
            'menu': self.site
        })


class Filter:
    def __init__(self, field_name, model, option="=",
                 template_name="adminlte/filters/select-filter.html"):
        self.model = model
        self.field_name = field_name
        self.option = option
        self.template_name = template_name
        self.field = self.get_field()

    def get_form(self):
        return modelform_factory(self.model, fields=(self.field_name,))

    def get_field(self):
        field = getattr(self.model, self.field_name, None)
        if not field:
            raise ValueError('field not exist')
        return field

    def get_model_choices(self):
        return [{'value': x.id, 'name': str(x)} for x in self.field.get_queryset()]

    def get_value(self, instance):
        return getattr(instance, self.field_name)

    def get_value_display(self, instance):
        try:
            return getattr(instance, 'get_%s_display' % self.field_name)
        except:
            return getattr(instance, self.field_name)

    def get_distinct_choices(self):
        return [{'value': self.get_value(x), 'name': self.get_value_display(x)}
                for x in self.model.objects.all().distinct(self.field_name).order_by(self.field_name)]

    def get_bool_choices(self):
        return [{'value': '1', 'name': 'Si'}, {'value': '0', 'name': 'No'}, ]

    def render(self):
        if isinstance(self.field, ForwardManyToOneDescriptor):
            return render_to_string(self.template_name, context={
                'field_name': self.field_name,
                'option': self.option,
                'choices': self.get_model_choices(),
                'table_field': '%s_id' % self.field_name,
                'form': self.get_form(),
            })
        if isinstance(self.field, DeferredAttribute):
            return render_to_string(self.template_name, context={
                'field_name': self.field_name,
                'option': '__icontains=',
                'table_field': self.field_name,
                'choices': self.get_distinct_choices(),
                'form': self.get_form(),
            })
        if isinstance(self.field, bool):
            return render_to_string(self.template_name, context={
                'field_name': self.field_name,
                'option': self.option,
                'table_field': self.field_name,
                'choices': self.get_bool_choices(),
                'form': self.get_form(),
            })


class Datatables(View):
    modal_width = 600
    list_template = "ajaxlte/datatables.html"
    form_template = "ajaxlte/datatables-modal.html"
    site = None
    model = None
    form = None
    buttons = [
        {
            'class': 'btn btn-success btn-perform',
            'perform': 'save',
            'callback': 'process_response',
            'icon': 'fa fa-save',
            'text': 'Guardar',
        },
    ]
    fieldsets = None
    fields = None
    media = None
    list_display = ()
    search_fields = ()
    list_filter = ()
    ordering = None
    include_request = False  # include request in the form init

    @classmethod
    def as_view(cls, **initkwars):
        return login_required(super().as_view(**initkwars), login_url=cls.site.login_url)

    def get_fields(self):
        field_names = []
        if self.fieldsets:
            for fieldset in self.fieldsets:
                field_names.extend(flatten(fieldset['fields']))
        elif self.fields and not self.fieldsets:
            return self.fields
        elif not self.fieldsets and not self.fields:
            return '__all__'
        return field_names

    def get_form(self):
        if not self.form:
            return modelform_factory(self.model, fields=self.get_fields())
        else:
            return self.form

    def get_buttons(self, request, instance=None):
        return self.buttons

    def html_form(self, instance, request, form, method):
        return render_to_string(self.form_template,
                                context={'opts': self.model._meta, 'fieldsets': self.fieldsets,
                                         'form': form, 'instance': instance, 'method': method,
                                         'buttons': self.get_buttons(request, instance=instance)},
                                request=request)

    def get_list_filters(self):
        return [Filter(x, self.model) for x in self.list_filter]

    def get_opts(self):
        return self.model._meta

    def get(self, request, **kwargs):
        return render(request, self.list_template, {
            'opts': self.get_opts(), 'list_display': self.list_display,
            'form': self.get_form(), 'form_template': self.form_template,
            'modal_width': self.modal_width, 'media': self.media,
            'list_filter': self.get_list_filters(), 'menu': self.site,
            'branding': self.site.branding(), **kwargs
        })

    def save_related(self, instance, request):
        pass

    @staticmethod
    def get_filters(filters):
        return [Q((field.split('=')[0], field.split('=')[1])) for field in filters.split('&')]

    def search_value(self, search_value):
        return [
            Q(('{}__icontains'.format(field), word))
            for word in search_value.split(' ')
            for field in self.search_fields
        ]

    def get_queryset(self, filters, search_value, request):
        queryset = self.model.objects.all()
        if not filters == "":
            queryset = queryset.filter(reduce(operator.and_, self.get_filters(filters)))
        if search_value:
            queryset = queryset.filter(reduce(operator.or_, self.search_value(search_value)))
        return queryset

    def get_ordered_queryset(self, filters, search_value, order, request):
        queryset = self.get_queryset(filters, search_value, request)
        return queryset.order_by(order)

    def get_data(self, start, per_page, filters, search_value, draw, order=None, request=None):
        queryset = self.get_queryset(filters, search_value, request)
        if order:
            queryset = self.get_ordered_queryset(filters, search_value, order, request)
        page = int(start / per_page) + 1
        paginator = Paginator(queryset, per_page)
        data = [x.to_json() for x in paginator.page(page).object_list]
        return {
            'draw': draw,
            'data': data,
            'recordsTotal': len(data),
            'recordsFiltered': queryset.count(),
        }

    def get_instance(self, request):
        try:
            return self.model.objects.get(id=int(request.POST.get('id')))
        except TypeError:
            return None
        except ValueError:
            return None

    @staticmethod
    def get_form_errors(form):
        return [{'key': f, 'errors': e.get_json_data()} for f, e in form.errors.items()]

    def process_request(self, request, method, instance=None):
        status = 200
        errors = []
        if instance:
            form = self.get_form()(request.POST, instance=instance)
        else:
            form = self.get_form()(request.POST)
        if form.is_valid():
            form.save()
            instance = form.instance
            self.save_related(instance=instance, request=request)
            form = self.get_form()(instance=instance)
            method = "POST"
        else:
            errors = self.get_form_errors(form)
            status = 203
        html_form = self.html_form(instance, request, form, method)
        return instance, html_form, errors, status

    @staticmethod
    def make_response(instance, html_form, errors, status):
        return JsonResponse({'instance': instance.to_json(),
                             'form': html_form, 'errors': errors}, encoder=Codec,
                            status=status)

    def post(self, request):
        status = 200
        errors = []
        instance = None
        html_form = ""
        if 'list' in request.POST:
            order = None
            start = int(request.POST.get('start', 0))
            draw = int(request.POST.get('draw', 0))
            per_page = int(request.POST.get('length', 10))
            search_value = request.POST.get('search[value]', None)
            filters = request.POST.get('filters', None)
            order_column = request.POST.get('order[0][column]', None)
            order_dir = request.POST.get('order[0][dir]', None)

            if order_column and order_dir:
                order = ''
                if order_dir == 'desc':
                    order += '-'
                list_display_column = self.list_display[int(order_column)]
                if type(list_display_column) == tuple:
                    order += list_display_column[1].replace('.', '__')
                else:
                    order += list_display_column.split('.')[0]

            return JsonResponse(self.get_data(start, per_page, filters, search_value, draw, order, request),
                                encoder=Codec)
        if 'open' in request.POST:
            instance = self.model.objects.get(id=int(request.POST.get('id')))
            if self.include_request:
                form = self.get_form()(instance=instance, request=request)
            else:
                form = self.get_form()(instance=instance)
            html_form = self.html_form(instance, request, form, 'POST')
        if 'save' in request.POST:
            instance = self.get_instance(request)
            instance, html_form, errors, status = self.process_request(request, 'POST', instance)
        return self.make_response(instance, html_form, errors, status)

    def put(self, request):
        status = 203
        instance = self.model()
        if self.include_request:
            form = self.get_form()(request=request)
        else:
            form = self.get_form()
        html_form = self.html_form(instance, request, form, 'PUT')
        errors = []

        if 'save' in request.PUT:
            instance, html_form, errors, status = self.process_request(request, 'PUT')
        return self.make_response(instance, html_form, errors, status)



from django import template
from django import forms
from django.conf import settings
from django.template.loader import get_template
from django.template import Context
from django.utils.safestring import mark_safe

register = template.Library()


@register.filter(name="datatables_label")
def datatables_label(form, fieldname):
    return form[fieldname].label


@register.filter(name="datatables_field")
def datatables_field(form, fieldname):
    try:
        return form[fieldname]
    except KeyError:
        return ""


@register.filter
def is_date(field):
    return isinstance(field.field.widget, forms.DateField)


@register.filter(name='as_inline_field')
def as_inline_field(field):
    if not isinstance(field, forms.BoundField) and settings.DEBUG:
        raise Exception('|as_inline_field got passed an invalid or inexistent field')

    attributes = {
        'field': field,
    }

    template_path = 'ajaxlte/datatables-field.html'
    template = get_template(template_path)

    c = Context(attributes).flatten()
    return template.render(c)


@register.filter(name="datatables_column")
def datatables_column(form, column):
    if isinstance(column, tuple):
        column_group = (column[0], column[1], 'false')
    else:
        name = column.split('.')[0]
        field = form[name]
        column_group = (field.label, column, 'true')

    return mark_safe('{title: "%s", data: "%s", orderable: %s},' % column_group)



# -*- coding:utf-8 -*-


def menu(request):
    return {
        'menu':
            {
                'pills': [
                    {
                        'name': "Test",
                        'links': [
                            {'name': 'test', 'url': ''},
                            {'name': 'test', 'url': '',
                             'has_treeview': True, 'tree': [
                                {'name': 'test', 'url': ''},
                            ]},
                        ]
                    },
                ]
            }
    }



# -*- coding:utf-8 -*-
import json
from django.db.models import Q, Max
import operator
import datetime
import decimal
from functools import reduce
from django.db.models.fields.files import ImageFieldFile, FileField, FieldFile
from django.contrib.contenttypes.models import ContentType


def get_code(entity, length=4):
    model = type(entity)
    code = ''
    sets = model.objects.filter(code__isnull=False)
    if sets:
        maxi = str(sets.aggregate(Max('code'))['code__max'])
        if maxi:
            consecutive = list(range(1, int(maxi)))
            busy = list(sets.values_list('code', flat=True))
            n = 0
            for l in busy:
                busy[n] = int(str(l))
                n += 1
            available = list(set(consecutive) - set(busy))
            if len(available) > 0:
                code = min(available)
            else:
                code = max(busy) + 1
    else:
        code = 1
    return str(code).zfill(length)


def get_number(entity, length=4):
    model = type(entity)
    code = ''
    sets = model.objects.filter(number__isnull=False)
    if sets:
        maxi = str(sets.aggregate(Max('number'))['number__max'])
        if maxi:
            consecutive = list(range(1, int(maxi)))
            busy = list(sets.values_list('number', flat=True))
            n = 0
            for l in busy:
                busy[n] = int(str(l))
                n += 1
            available = list(set(consecutive) - set(busy))
            if len(available) > 0:
                code = min(available)
            else:
                code = max(busy) + 1
    else:
        code = 1
    return str(code).zfill(length)


def json_object(instance, model):
    if instance:
        return instance.to_json()
    else:
        return model().to_json()


def json_choice(choice_name, instance):
    try:
        return {'id': getattr(instance, choice_name), 'name': getattr(instance, 'get_%s_display' % choice_name)()}
    except:
        return None


class Codec(json.JSONEncoder):
    def default(self, obj):
        if hasattr(obj, 'isoformat'):
            return obj.isoformat()
        elif isinstance(obj, decimal.Decimal):
            return float(obj)
        elif isinstance(obj, datetime.datetime):
            return obj.strftime('%d/%m/Y %H:%M:%S')
        elif isinstance(obj, datetime.time):
            return obj.strftime('H:%M:%S')
        elif isinstance(obj, datetime.date):
            return obj.strftime('%d/%m/%Y')
        elif isinstance(obj, FieldFile):
            try:
                return obj.url
            except:
                return 'null'
        elif isinstance(obj, FileField):
            try:
                return obj.url
            except:
                return 'null'
        elif isinstance(obj, ImageFieldFile):
            try:
                return obj.url
            except:
                return 'null'
        elif obj == None:
            return 'null'
        else:
            return json.JSONEncoder.default(self, obj)


class Filter(object):
    app_label = None
    model_name = None
    model = None

    def __init__(self, app_label, model_name):
        self.app_label = app_label
        self.model_name = model_name
        self.model = ContentType.objects.get(app_label=app_label, model=model_name).model_class()

    def get_instance(self, pk):
        return self.model.objects.get(pk=pk)

    @staticmethod
    def _like(sentence, field_name, separator=" ", filters=[]):
        for word in sentence.split(separator):
            filters.append(('{}__icontains'.format(field_name.replace('__like', '')), word))
        return filters

    def format(self, filters):
        final_filter = []
        for f in filters:
            if f[0].find('__like') > -1:
                final_filter = self._like(f[1], f[0], filters=final_filter)
            else:
                final_filter.append(f)
        return [Q(x) for x in final_filter]

    def filter_by_json(self, filters=None, op=operator.and_):
        if filters:
            final_filter = []
            for k, v in json.loads(str(filters).replace("'", "\"")).items():
                final_filter.append((str(k), str(v)))
            return self.model.objects.filter(reduce(op, self.format(final_filter)))
        else:
            return self.model.objects.all()

    def filter_by_list(self, filters=[], op=operator.and_, extra=None):
        if len(filters) > 0:
            qs = self.model.objects.filter(reduce(op, self.format(filters)))
            if extra:
                qs = qs.filter(reduce(operator.and_, self.format(extra)))
            return qs
        else:
            return self.model.objects.all()



from .utils import json, Codec, Filter
from django.http import HttpResponse
from django.forms import modelform_factory
from django.template.loader import render_to_string
import operator


def load_module(mod, cls):
    mod = __import__(mod, fromlist=[cls])
    return getattr(mod, cls)


def get_object(request):
    instance = Filter(app_label=request.POST.get('app_label'),
                      model_name=request.POST.get('model')
                      ).get_instance(request.POST.get('id'))
    return HttpResponse(json.dumps(instance.to_json(), cls=Codec), content_type='application/json')


def object_update(request):
    result = ""
    instance = Filter(app_label=request.POST.get('app_label'),
                      model_name=request.POST.get('model')
                      ).get_instance(request.POST.get('id'))
    data = request.POST.get('data', None)
    if data:
        for k, v in json.loads(str(data).replace("'", "\"")).items():
            setattr(instance, k, v)
        instance.save()
    return HttpResponse(json.dumps({'result': result, 'intance': instance.to_json()}, cls=Codec),
                        content_type='application/json')


def object_view(request):
    instance = Filter(app_label=request.POST.get('app_label'),
                      model_name=request.POST.get('model')
                      ).get_instance(request.POST.get('id'))
    try:
        result = getattr(instance, request.POST.get('view'))(request)
    except:
        result = getattr(instance, request.POST.get('view'))()
    return HttpResponse(result)


def object_execute(request):
    instance = None
    f = Filter(app_label=request.POST.get('app_label'),
                      model_name=request.POST.get('model')
                      )
    id = request.POST.get('id')
    if id:
        instance = f.get_instance(id)
    else:
        instance = f.model
    try:
        result = getattr(instance, request.POST.get('view'))(request)
    except:
        try:
            result = getattr(instance, request.POST.get('view'))()
        except:
            result = str(getattr(instance, request.POST.get('view')))
    return HttpResponse(json.dumps(result, cls=Codec), content_type='application/json')


def get_collection(request):
    queryset = Filter(app_label=request.POST.get('app_label', request.GET.get('app_label')),
                      model_name=request.POST.get('model', request.GET.get('model'))
                      ).filter_by_json(request.POST.get('filters', request.GET.get('filters')))
    return HttpResponse(json.dumps([x.to_json() for x in queryset], cls=Codec),
                        content_type='application/json')


def get_datatables(request):
    queryset = Filter(app_label=request.POST.get('app_label', request.GET.get('app_label')),
                      model_name=request.POST.get('model', request.GET.get('model'))
                      ).filter_by_json(request.POST.get('filters', request.GET.get('filters')))
    return HttpResponse(json.dumps({'data': [x.to_json() for x in queryset]}, cls=Codec),
                        content_type='application/json')


def autocomplete(request):
    result = []
    columns = request.GET.get('column_name').split(",")
    value = request.GET.get('column_value')
    columns = [('{}__like'.format(column), request.GET.get('term')) for column in columns]
    filters = request.GET.get('filters', [])
    if filters:
        filters = filters.split(",")
        filters = [tuple(x.split("=")) for x in filters]
    queryset = Filter(app_label=request.GET.get('app_label'),
                      model_name=request.GET.get('model')
                      ).filter_by_list(columns, operator.or_, filters)
    for q in queryset:
        result.append({'obj': q.to_json(),
                       'label': str(q),
                       'value': q.to_json()[value]})
    return HttpResponse(json.dumps(result, cls=Codec), content_type="application/json")


def get_html_form(request):
    resp = {}
    if request.method == "GET":
        data = {"method": "POST"}
        filter = Filter(app_label=request.GET.get('app_label'),
                        model_name=request.GET.get('model'))
        data['fields'] = request.GET.get('fields')
        data['callback'] = str(request.GET.get('callback', ""))
        data['params'] = str(request.GET.get('params', ""))
        actions = request.GET.get('action')
        _form = request.GET.get('form')
        if actions:
            data['action'] = actions
        else:
            data['action'] = '/admin/ajax/get_html_form/'
        data['app_label'] = filter.app_label
        data['model'] = filter.model_name
        id = request.GET.get('id', None)
        if _form:
            form = load_module(_form.split("-")[0], _form.split("-")[1])
        else:
            form = modelform_factory(filter.model, fields=data['fields'].split("-"))
        if id:
            form = form(instance=filter.get_instance(id))
            data['id'] = id
        else:
            form = form()
        data['form'] = form
        return HttpResponse(render_to_string("ajax/form.html", data, request))
    if request.method == "POST":
        filter = Filter(app_label=request.POST.get('app_label'),
                        model_name=request.POST.get('model'))
        form = modelform_factory(filter.model, fields=request.POST.get('fields').split("-"))
        id = request.POST.get('id', None)
        if id:
            form = form(request.POST, request.FILES or None, instance=filter.get_instance(id))
        else:
            form = form(request.POST, request.FILES)
        if form.is_valid():
            form.save()
            obj = form.instance
            resp = {"result": "actualizado", "object": obj.to_json()}
        else:
            err = ""
            for e in form.errors:
                err += "error en el campo " + str(e)
            resp = {'error': err}
    return HttpResponse(json.dumps(resp, cls=Codec), content_type="application/json")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import print_function
from setuptools import setup, find_packages
import os
import re
import sys


def new_version(mo):
    x = int(mo.group(1).split(".")[2])
    y = int(mo.group(1).split(".")[1])
    z = int(mo.group(1).split(".")[0])
    if x < 9:
        x += 1
    elif x == 9 and y < 9:
        x = 0
        y += 1
    elif x == 9 and y == 9:
        x = 0
        y = 0
        z += 1

    NEW_VERSION = "__version__ = '%s.%s.%s'" % (z, y, x)
    open(VERSION_FILE, "w").write(NEW_VERSION)
    return "%s.%s.%s" % (z, y, x)


def get_version(mo):
    x = int(mo.group(1).split(".")[2])
    y = int(mo.group(1).split(".")[1])
    z = int(mo.group(1).split(".")[0])
    return "%s.%s.%s" % (z, y, x)


VERSION_FILE = 'ajaxlte/__init__.py'
version_text = open(VERSION_FILE, "rt").read()
VSRE = r"^__version__ = ['\"]([^'\"]*)['\"]"
mo = re.search(VSRE, version_text, re.M)
if not mo:
    raise RuntimeError(
        "Unable to find version string in %s." % (VERSION_FILE,))

name = 'django-ajaxlte'
package = 'ajaxlte'
description = 'Genrerate adminlte cruds for models'
url = 'https://github.com/xangcastle/django_ajaxlte/'
author = 'Cesar Abel Ramirez'
author_email = 'xangcastle@gmail.com'
license = 'BSD'
install_requires = [
    'Django>=2.0.0',
    'django-crispy-forms>=1.7.0',
    'django-mathfilters>=0.4.0'
]

version = get_version(mo)

if sys.argv[-1] == 'publish':
    version = new_version(mo)
    args = {'version': version}
    os.system("python setup.py sdist bdist_wheel")
    os.system("twine upload dist/django-ajaxlte-%(version)s*" % args)
    print("Creating git tag the version now:")
    os.system("git tag -a %(version)s -m 'version %(version)s'" % args)
    os.system("git push --tags")
    sys.exit()


def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname), "r").read()


setup(
    name=name,
    version=version,
    url=url,
    author=author,
    author_email=author_email,
    license=license,
    packages=['ajaxlte', 'ajaxlte.templatetags'],
    include_package_data=True,
    description=description,
    long_description=read('README.md'),
    long_description_content_type="text/markdown",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: Apache Software License",
        "Programming Language :: Python",
        "Operating System :: OS Independent",
        "Topic :: Software Development :: Libraries",
        "Topic :: Utilities",
        "Environment :: Web Environment",
        "Framework :: Django",
    ],
    install_requires=install_requires
)






